#This code will read in the white ash microsat dataset (excluding CBG samples)
#and subsample it for different sampling strategies

#More specifically it will read in the genotype files, 
#categorize alleles into 8 categories based on commonness and spatial occurrence
#perform a number of (num_reps) replicates in which
#	a list of POPS_TO_SAMP is chosen, defining the populations to sample for that rep
#		there is one element in this list for each num pops/ num samps combination
#		list_pops_sampled[[nrep]] will record these sampled populations for later reference
#	a list of NUM_SAMPS is chosen- this is the number of trees in each of the populations
#		this is set at an increasing number (1,2,3...) but if the population is smaller than T trees
#		the sample number is reduced to the population size
#	the sampling is performed using sample.pop
#	the results are recorded in summ_results array in which dimensions are 
#		[sample strategies, plants/pops/allele categories, "scenarios", replicates]
#		only one scenario is implemented for now- assumes random choice of trees
#	
#Then in the next section it identifies the best populations (those making the highest diversity).
#	For each sampling strategy (ss) it sorts and chooses the rep with highest diversity, and 
#	pulls the populations from list_pops_sampled already recorded
#And it then performs another round of sampling with just those exact populations but sampling
#	many (num_rep) combinations of individuals within them 
#	(to show they are really best and its not a fluke that they are the best populations)
#		Currently it uses the exact "top" population, but can be modified to choose one of top five
#	This is saved as summ_results_2- all sampled from the optimal population set for each ss, but 
#	resampled many times for different individuals
#	we will then compare the mean for the optimal sampling to the mean from random sampling


setwd("C:/Users/shoban.DESKTOP-DLPV5IJ/Dropbox/Projects/Fant_USFS_white_ash/")
setwd("C:/Users/shoban/Dropbox/Projects/Fant_USFS_white_ash/")
setwd("/home/user/Dropbox/Projects/Fant_USFS_white_ash/")
library(adegenet)
library(parallel);	library(doParallel) #will load foreach
source("Fa_sample_funcs.R")
colMax <- function(data) sapply(data, max, na.rm = TRUE)

	###########EDIT TO SAMPLE.POP FUNCTION IN ADEGENET#############################
#Need to add na.rm=T because of individuals with 000000.  When calling @tab (a matrix of chosen individuals by alleles)
#	individuals usually get a 0, 1, or 2 but if their genotype is 000000
#	then a NA will be listed instead for that individual and that allele
#	and if this is then summed by columns (alleles), a NA is passed overall for all the individuals in the sample
#colSums(Fa_genind_sep[[POPS_TO_SAMP[[40]][1]]]@tab) #this is all individuals in the population
#colSums(Fa_genind_sep[[POPS_TO_SAMP[[40]][1]]]@tab,na.rm=T)
sample.pop<-function(genind_obj,vect_pop_ID,vect_samp_sizes){
        for (p in 1:length(vect_pop_ID))
        alleles[p,]<-colSums(genind_obj[[vect_pop_ID[p]]]@tab[sample(1:nrow(genind_obj[[vect_pop_ID[p]]]@tab), vect_samp_sizes[p]),],na.rm=T)
        alleles
}       

	############################################
	#---SETUP- READ FILES, DECLARE VARIABLES---#
	############################################
	Fa_genind<-read.genepop("whiteash_genepop.gen",ncode=3)
	Fa_genpop<-genind2genpop(Fa_genind)
	Fa_genind_sep<-seppop(Fa_genind)
	n_pops<-length(levels(Fa_genind@pop))
	n_total_indivs<- length(Fa_genind@tab[,1])
	n_ind_p_pop<-table(Fa_genind@pop)
	allele_freqs<-colSums(Fa_genpop@tab)/(n_total_indivs*2)	
	
	#NOTE regional alleles not used in this manuscript- used in other projects. Also I used the second def'n of loc common
	list_allele_cat<-c("global","glob_v_com","glob_com","glob_lowfr","glob_rare","reg_rare","loc_com_d1","loc_com_d2","loc_rare")
		
	######NEED TO SAVE THESE CONDITIONS TO A PARAMS FILE!!!!
	num_scen<-1; scen<-1
	num_reps<-100000
	max_num_trees<-20; max_num_pops<-10
	#Sample plants from 2 to max_num_trees, l refers to length; pops from 1 to max_num_pops
	l_plt_smp<-length(2:max_num_trees)
	save(num_scen,scen,num_reps,max_num_trees,max_num_pops,l_plt_smp,file="params_run.Rda")
	write(paste("num_scen=",num_scen,"\nscen=",scen,"\nnum_reps=",num_reps,"\nmax_num_trees=",max_num_trees,
				"\nmax_num_pops=",max_num_pops,"\nl_plt_smp=",l_plt_smp),file="list_params.txt")
	
	n_pops_to_samp<- rep(1:max_num_pops, each=l_plt_smp)
	type_samp<-c("random")
	region_makeup<-vector("list",2); region_makeup[[1]]<-1:10; region_makeup[[2]]<-11:17
		
	#places to put results
	summ_results<-array(dim=c(max_num_pops*(max_num_trees-1)+1+2,11,num_scen,num_reps))
	#This is a list of the actual populations sampled, to keep a record for determining "best" reps
	list_pops_sampled<-list(); list_num_trees_sampled<-list()
	
	#######################################################
	#----DETERMINE WHAT ALLELES FALL IN WHAT CATEGORIES---#
	#######################################################
	allele_cat<-get.allele.cat(Fa_genpop, region_makeup, 2, n_ind_p_pop)	
	#glob; 	glob_v_com; 	glob_com;	glob_lowfr;		glob_rare
	#reg_com_int;	loc_com_d1;		loc_com_d2;		loc_rare
	#!!!RESULT!!!- HOW MANY ALLELES IN EACH CATEGORY
	for (i in 1:9) print(length(allele_cat[[i]]))

	
	
	#######################
	#--SUBSAMPLE SECTION--#
	#######################


cl <- makeCluster(28) # create a cluster with X cores
registerDoParallel(cl) # register the cluster
list_results <- foreach (nrep = 1:num_reps, .packages="adegenet")	%dopar% {

	### DECLARE POPULATIONS AND INDIVIDUALS SAMPLED
	POPS_TO_SAMP<-list();	this_slot<-1
	#Actually choosing which populations to sample in this rep (t refers to the spatial patterns)
	#can add different t's to sample with more types of sampling, i.e. non random- see Kew excelsior project
	for (t in 1:length(type_samp)) 
		for (p in 1:length(unique(n_pops_to_samp))) {
			if (t==1)	the_pops<-sample(1:n_pops, unique(n_pops_to_samp)[p])
				#For each set of populations chosen, write that set over and over for as many "plant" sampling options as there are
			for (x in 1:l_plt_smp) { POPS_TO_SAMP[[this_slot]]<-the_pops; this_slot=this_slot+1}
	}
	N_SAMPS_P_POP<-as.list(rep(2:max_num_trees,(length(type_samp)*length(unique(n_pops_to_samp)))))
	#necessary for formatting
	for (i in 1:length(POPS_TO_SAMP)) 
		N_SAMPS_P_POP[[i]]<-unlist(rep(N_SAMPS_P_POP[i],length(POPS_TO_SAMP[[i]])))
	#If trying to sample more individuals in population than exist, reduce sample size to pop size
	for (p in 1:length(POPS_TO_SAMP))	for(this_p in 1:length(POPS_TO_SAMP[[p]]))
			if ((table(Fa_genind@pop)[POPS_TO_SAMP[[p]][this_p]]) < (N_SAMPS_P_POP[[p]][this_p]))
				N_SAMPS_P_POP[[p]][this_p] <- table(Fa_genind@pop)[POPS_TO_SAMP[[p]][this_p]]
	
	#This will add additional POPS_TO_SAMP elemnts for the treated and untreated populations
	p<-length(summ_results[,1,1,1])
	POPS_TO_SAMP[[p-1]]<-c(1:6,9,10,13,15); N_SAMPS_P_POP[[p-1]]<-table(Fa_genind@pop)[c(1:6,9,10,13,15)]
	POPS_TO_SAMP[[p-2]]<-c(7,8,11,12,14,16,17); N_SAMPS_P_POP[[p-2]]<-table(Fa_genind@pop)[c(7,8,11,12,14,16,17)]

	#IMPORTANT RECORD the populations sampled for later because POPS_TO_SAMP gets wiped every rep
	list_pops_sampled[[nrep]]<-POPS_TO_SAMP; list_num_trees_sampled<-N_SAMPS_P_POP
	
	#actually DO the sampling
	for (samp in 1:length(POPS_TO_SAMP)){
		alleles<-matrix(nrow=length(POPS_TO_SAMP[[samp]]),ncol=length(allele_freqs))
		alleles<-sample.pop(Fa_genind_sep,POPS_TO_SAMP[[samp]],N_SAMPS_P_POP[[samp]])
		
		#record number of pops and number of trees
		summ_results[samp,1:2,scen,nrep]<-c(sum(N_SAMPS_P_POP[[samp]]), length(N_SAMPS_P_POP[[samp]]))
		
		#this will record each time an allele is CAUGHT in one of our sampled populations
		#By summing the columns for alleles, focusing on those in allele cat l, and counting how many are greater than 0
		for (l in 1:length(allele_cat)) summ_results[samp,l+2,scen,nrep]<-sum(colSums(alleles,na.rm=T)[allele_cat[[l]]]>0)
	}
	#this will record the number of alleles that EXISTS for each category in last row
	for(l in 1:length(allele_cat)) summ_results[length(summ_results[,1,1,1]),l+2,scen,nrep]<-length(allele_cat[[l]])
		
	#RETURN VALUE FOREACH
	list(summ_results[,,1,nrep],list_pops_sampled[[nrep]])
		
}
for (nrep in 1:num_reps) summ_results[,,1,nrep] <- list_results[[nrep]][[1]]
for (nrep in 1:num_reps) list_pops_sampled[[nrep]] <- list_results[[nrep]][[2]]

	stopCluster(cl)
	
save(summ_results,file="summ_results.R"); save(list_pops_sampled,file="list_pops_sampled.R")



	#########################################################
	#--FINDING BEST COMBINATIONS OF POPULATIONS BY SORTING--#
	#########################################################


setwd("C:/Users/shoban.DESKTOP-DLPV5IJ/Dropbox/Projects/Fant_USFS_white_ash/")
setwd("C:/Users/shoban/Dropbox/Projects/Fant_USFS_white_ash/")
setwd("/home/user/Dropbox/Projects/Fant_USFS_white_ash/")
load(file="summ_results.R"); load(file="list_pops_sampled.R")
#remove rows for total and for "treated"/"untreated" and store as summ_results_r for reduced
summ_results_r<-array(dim=c(190,dim(summ_results)[2],1,dim(summ_results)[4])); summ_results_r[1:190,,1,]<-summ_results[1:190,,1,]

POPS_a<-list()
topx<-5
#do this for each allele category
for (foc_all in 3:11){

	all_top_pops<-list();	this_top_pops<-list()
	#To find the populations who best capture genetic diversity for the foc_all category (all_top_pops)
	#Go through each of the sampling strategies (ss) (all rows in summ_results_r) (this refers to this ss)
	for (ss in 1:(dim(summ_results_r)[1])){
		#the top X values of proportion of genetic diversity captured for that sampling strategy across all reps
		top_values<-tail(sort(summ_results_r[ss,foc_all,1,]),topx)
		
		#go through those top diversity values and "match" them to the rep number that produced them (temp_pop_rep)
		#then find the actual populations from that rep using (list_pops_sampled)
		#if multiple reps produced that same top genetic value, sample is used to pick a random one
		for (i in 1:length(top_values)){
			temp_pop_rep<-which(summ_results_r[ss,foc_all,1,]==top_values[i])
			if(length(temp_pop_rep)==1) this_top_pops[[i]]<-list_pops_sampled[[temp_pop_rep]][[ss]]
			else this_top_pops[[i]]<-list_pops_sampled[[sample(temp_pop_rep,1)]][[ss]]
		}
		#print(sort(table(top_pops)))
		all_top_pops[[ss]]<-this_top_pops
	}
	
	#Sort all top pops so its easier to look at (just for me)
#	for (l in 1:length(all_top_pops)) for (w in 1:length(all_top_pops[[l]])) 
#		all_top_pops[[l]][[w]]<-sort(all_top_pops[[l]][[w]])
#	sort(table(as.character(all_top_pops[[10]])))

	#This will take the top pop'n and resample it with different indiv's chosen and see where that outcome compares
	POPS_TO_SAMP_2<-list()
	for (i in 1:length(all_top_pops)) POPS_TO_SAMP_2[[i]]<-(all_top_pops[[i]][[1]]) 
	#NOTE: MODIFICATION: instead of 1 use sample(5,1) to identify pop'n combinations resulting in
	#top 5 diversity outcomes, and randomly pick one of them to resample
 	N_SAMPS_P_POP_2<-as.list(rep(2:max_num_trees,(length(type_samp)*length(unique(n_pops_to_samp)))))
	for (i in 1:length(POPS_TO_SAMP_2)) 
		N_SAMPS_P_POP_2[[i]]<-unlist(rep(N_SAMPS_P_POP_2[i],length(POPS_TO_SAMP_2[[i]])))
	#If trying to sample more individuals in population than exist, reduce sample size to pop size
		for (p in 1:length(POPS_TO_SAMP_2))	for(this_p in 1:length(POPS_TO_SAMP_2[[p]]))
				if ((table(Fa_genind@pop)[POPS_TO_SAMP_2[[p]][this_p]]) < (N_SAMPS_P_POP_2[[p]][this_p]))
					N_SAMPS_P_POP_2[[p]][this_p] <- table(Fa_genind@pop)[POPS_TO_SAMP_2[[p]][this_p]]
		POPS_a[[foc_all]]<-POPS_TO_SAMP_2
	
	summ_results_2<-array(dim=c(max_num_pops*(max_num_trees-1)+3,11,num_scen,num_reps))
					
	cl <- makeCluster(28) # create a cluster with X cores
	registerDoParallel(cl) # register the cluster
	list_results <- foreach (nrep = 1:num_reps, .packages="adegenet")	%dopar% {
		for (samp in 1:length(POPS_TO_SAMP_2)){
			alleles<-matrix(nrow=length(POPS_TO_SAMP_2[[samp]]),ncol=length(allele_freqs))
			alleles<-sample.pop(Fa_genind_sep,POPS_TO_SAMP_2[[samp]],N_SAMPS_P_POP_2[[samp]])
			summ_results_2[samp,1:2,scen,nrep]<-c(sum(N_SAMPS_P_POP_2[[samp]]),length(N_SAMPS_P_POP_2[[samp]]))
			for (l in 1:length(allele_cat)) summ_results_2[samp,l+2,scen,nrep]<-sum(colSums(alleles,na.rm=T)[allele_cat[[l]]]>0)
		}
		for(l in 1:length(allele_cat)) summ_results_2[length(summ_results_2[,1,1,1]),l+2,scen,nrep]<-length(allele_cat[[l]])

		#RETURN VALUE FOREACH
		summ_results_2[,,1,nrep]
	}
	stopCluster(cl)

	for (nrep in 1:num_reps) summ_results_2[,,1,nrep] <- list_results[[nrep]]
	
	for (n in 1:num_reps) summ_results_2[,3:11,1,n]<-t(t(summ_results_2[,3:11,1,n])/summ_results_2[length(summ_results_2[,1,1,1]),3:11,1,n])	
	save(summ_results_2,file=paste("summ_results_",foc_all,"top1.R",sep=""))
}
save(POPS_a,file="POPS_a.R")
